import re
from django.db.models import Q
from rest_framework.status import HTTP_200_OK
from rest_framework.utils import json

from interceptor.models import InterceptorSession


class SessionNotFoundError(Exception):
    pass


class BaseInterceptor:
    _meta = {}
    _data = {}
    _files = []
    path = None
    headers = None
    method = ''
    content_type = ''
    

    safe_meta = [
        'HTTP_USER_AGENT',
        'HTTP_ACCEPT',
        'HTTP_CACHE_CONTROL',
        'HTTP_ACCEPT_ENCODING',
        'HTTP_CONNECTION',
        'REMOTE_ADDR',
        'PATH_INFO',
        'REQUEST_METHOD',
        'CONTENT_LENGTH',
        'REMOTE_HOST',
        'CONTENT_TYPE'
    ]


class HttpInterceptor(BaseInterceptor):

    session = None

    def __init__(self, request, user=None):
        self.user = user
        self.method = request.method
        self.headers = request.headers

        self.set_path(request.META.get('PATH_INFO'))
        self.set_meta(request.META)

        self.extract_data_and_files(request.data, request.FILES)
        self.extract_content_type()
        
    def set_path(self, path):
        self.path = path.replace('/interceptor', '')
    
    def set_meta(self, meta):
        self._meta = {
            key: value for (key, value) in meta.items() if (
                    isinstance(value, str) and key in self.safe_meta
            )
        }

        return self._meta
    
    def extract_data_and_files(self, data, files):
        self._data = {key: value for key, value in data.items() if key not in files.keys()}
        self._files = files
        
    def extract_content_type(self):
        self.content_type = self._meta.get('CONTENT_TYPE').split(';')[0]

    def authenticate(self):
        return self.user

    def can_perform_creation(self):
        return True

    @property
    def data(self):
        return self._data

    @property
    def files(self):
        return self._files

    @property
    def meta(self):
        return self._meta

    def response_data(self):
        return {
            "data": {'status': 'You have sent a request, but we encourage you to create your own sessions!'},
            "status": HTTP_200_OK,
            "headers": {}
        }


class HttpSessionInterceptor(HttpInterceptor):
    session = None

    def __init__(self, request, session_name, user=None):
        query = Q(short_name=session_name)
        if user:
            query &= Q(user=user)

        session = InterceptorSession.objects.filter(query)

        if not session.exists():
            raise SessionNotFoundError()

        self.session = session.first()
        super(HttpSessionInterceptor, self).__init__(request, user=user)

    def authenticate(self):
        token = self.headers.get('AUTHORIZATION', '')
        if len(token.split(' ')) > 1:
            token = token.split(' ')[-1]

        if token and self.session.session_token == token:
            self.user = self.session.user
            return self.user

    def can_perform_creation(self):
        if self.session.requires_authentication and not self.user:
            return False
        if self.session.requires_authentication and self.session.user != self.user:
            return False
        return True

    def set_path(self, path):
        session_name = self.session.short_name
        regex = rf'.*\/{session_name}(?<!\/)'
        self.path = re.sub(regex, '', path)

    def response_data(self):
        mocks = self.session.mocks
        if mocks.filter(path=self.path, method=self.method.lower()).exists():
            mock = mocks.filter(path=self.path, method=self.method.lower()).first()
            headers = '{}' if not mock.response_headers else json.loads(mock.response_headers)

            return {
                "data": json.loads(mock.response_body),
                "status": mock.status_code,
                "headers": headers,
                "mock": mock
            }

        return super(HttpSessionInterceptor, self).response_data()



# Generated by Django 3.0.5 on 2020-05-18 05:29

from django.db import migrations, models
import django.db.models.deletion
import interceptor.validators


class Migration(migrations.Migration):

    dependencies = [
        ('interceptor', '0009_interceptormockresponse'),
    ]

    operations = [
        migrations.AlterField(
            model_name='interceptedfile',
            name='request',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='files', to='interceptor.InterceptedRequest'),
        ),
        migrations.AlterField(
            model_name='interceptedrequest',
            name='session',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='requests', to='interceptor.InterceptorSession'),
        ),
        migrations.AlterField(
            model_name='interceptormockresponse',
            name='response_body',
            field=models.TextField(validators=[interceptor.validators.JSONFormatValidator()]),
        ),
        migrations.AlterField(
            model_name='interceptormockresponse',
            name='response_headers',
            field=models.TextField(blank=True, null=True, validators=[interceptor.validators.JSONFormatValidator(allow_null=True)]),
        ),
        migrations.AlterField(
            model_name='interceptormockresponse',
            name='session',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='mocks', to='interceptor.InterceptorSession'),
        ),
    ]



import json

from django.core.exceptions import ValidationError
from django.utils.deconstruct import deconstructible


@deconstructible
class JSONFormatValidator:
    def __init__(self, allow_null=False):
        self.allow_null = allow_null

    def __call__(self, value):
        if not value and self.allow_null:
            return
        try:
            json.loads(value)
        except json.decoder.JSONDecodeError:
            raise ValidationError('Value is not a valid JSON string.')



import json
from rest_framework.response import Response
from rest_framework.status import HTTP_401_UNAUTHORIZED, HTTP_200_OK
from rest_framework.views import APIView

from interceptor.interceptor import HttpSessionInterceptor, HttpInterceptor, SessionNotFoundError
from interceptor.models import InterceptedRequest, InterceptedFile


class HTTPInterceptorView(APIView):
    """
    HTTPInterceptorView is a basic view that intercepts any request without having an existing session
    """

    interceptor_class = HttpInterceptor
    interceptor = None

    def __getattr__(self, item):
        if item in self.http_method_names:
            return self.intercept
        try:
            return super(HTTPInterceptorView, self).__getattr__(item)
        except AttributeError:
            raise AttributeError(item)

    def options(self, request, *args, **kwargs):
        # TODO: Check cors and if cors do not intercept here
        cors = False
        if cors:
            if self.metadata_class is None:
                return self.http_method_not_allowed(request, *args, **kwargs)
            data = self.metadata_class().determine_metadata(request, self)
            return Response(data, status=HTTP_200_OK)
        else:
            return self.intercept(request, *args, **kwargs)

    def get_interceptor_class(self):
        assert self.interceptor_class is not None, (
                "'%s' should either include a `interceptor_class` attribute, "
                "or override the `get_interceptor_class()` method."
                % self.__class__.__name__
        )

        return self.interceptor_class

    def get_interceptor_class_kwargs(self, **kwargs):
        kwargs = kwargs or {}

        kwargs.update({
            'request': self.request,
        })
        return kwargs

    def get_interceptor(self, **kwargs):
        kwargs = self.get_interceptor_class_kwargs(**kwargs)
        interceptor_class = self.get_interceptor_class()
        return interceptor_class(**kwargs)

    def initial(self, request, *args, **kwargs):
        self.interceptor = self.get_interceptor(
            user=request.user if request.user.is_authenticated else None
        )
        return super(HTTPInterceptorView, self).initial(request, *args, **kwargs)

    def build_response(self, request):
        response = self.interceptor.response_data()

        return Response(
            data=response.get('data'),
            status=response.get('status'),
            headers=response.get('headers')
        )

    def create_intercepted_file(self, intercepted_request, param, file):
        instance = InterceptedFile.objects.create(
            request=intercepted_request,
            parameter=param,
            filename=file.name,
            size=file.size
        )

        return instance

    def intercept(self, request, *args, **kwargs):
        """
        Intercepts all request to main URL and save it on database.
        """
        if self.interceptor.can_perform_creation():

            intercepted_request = InterceptedRequest.objects.create(
                path=self.interceptor.path,
                method=request.method,
                params=json.dumps(request.query_params),
                data=json.dumps(self.interceptor.data),
                metadata=json.dumps(self.interceptor.meta),
                headers=json.dumps(dict(request.headers)),
                content_type=self.interceptor.content_type,
                session=self.interceptor.session
            )

            self.create_intercepted_files(intercepted_request, self.interceptor.files)

            return self.build_response(intercepted_request)

        else:
            return Response(data={'error': 'Unauthorized'}, status=HTTP_401_UNAUTHORIZED)

    def create_intercepted_files(self, request, files):
        for key, file in files.items():
            self.create_intercepted_file(request, key, file)


class HTTPSessionInterceptorView(HTTPInterceptorView):

    interceptor_class = HttpSessionInterceptor

    def initial(self, request, *args, **kwargs):
        try:
            return super(HTTPSessionInterceptorView, self).initial(request, *args, **kwargs)
        except SessionNotFoundError:
            raise Response(status=HTTP_401_UNAUTHORIZED)

    def get_interceptor_class_kwargs(self, **kwargs):
        session_name = self.kwargs.pop('session_name', None)
        kwargs = super(HTTPSessionInterceptorView, self).get_interceptor_class_kwargs(**kwargs)
        kwargs.update({'session_name': session_name})
        return kwargs

    def perform_authentication(self, request):
        """
        Overrides if authentication will be performed or overrided after request sending, for example a token
        in the session to override user in the request.
        """
        user = self.interceptor.authenticate()
        if user is not None:
            request.user = user

    def create_intercepted_file(self, intercepted_request, param, file, session=None):
        instance = super(HTTPSessionInterceptorView, self).create_intercepted_file(
            intercepted_request,
            param,
            file
        )

        if session and session.saves_files:
            instance.file = file
            instance.save()

    def create_intercepted_files(self, request, files):
        for key, file in files.items():
            self.create_intercepted_file(request, key, file, session=self.interceptor.session)

    def build_response(self, request):
        response = self.interceptor.response_data()

        request.matched_response = response.get('mock')
        request.save()

        return Response(
            data=response.get('data'),
            status=response.get('status'),
            headers=response.get('headers')
        )



# Generated by Django 3.0.5 on 2020-05-14 06:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('interceptor', '0005_auto_20200514_0617'),
    ]

    operations = [
        migrations.AddField(
            model_name='interceptedrequest',
            name='path',
            field=models.CharField(default='/', max_length=255),
        ),
        migrations.AddField(
            model_name='interceptedrequest',
            name='session',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='interceptor.InterceptorSession'),
        ),
    ]



from django.conf import settings
from rest_framework import authentication
from rest_framework.mixins import ListModelMixin, RetrieveModelMixin
from rest_framework.pagination import PageNumberPagination
from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import GenericViewSet, ModelViewSet

from interceptor.api.permissions import SessionOwnerPermission
from interceptor.api.serializers import InterceptedRequestModelSerializer, InterceptorSessionSerializer
from interceptor.models import InterceptedRequest, InterceptorSession


class AuthenticatorMixin:
    authentication_classes = [authentication.SessionAuthentication, authentication.BasicAuthentication]

    def get_authenticators(self):
        authenticators = super(AuthenticatorMixin, self).get_authenticators()
        if 'rest_framework.authtoken' in settings.INSTALLED_APPS:
            authenticators.append(authentication.TokenAuthentication)
        return authenticators


class RendererHTMLOnDebugMixin:
    def get_renderers(self):
        renderers = super(RendererHTMLOnDebugMixin, self).get_renderers()
        if not settings.DEBUG:
            renderers = [r for r in renderers if r.media_type != 'text/html']
        return renderers


class InterceptedRequestModelListViewset(
    ListModelMixin,
    RetrieveModelMixin,
    AuthenticatorMixin,
    RendererHTMLOnDebugMixin,
    GenericViewSet
):
    queryset = InterceptedRequest.objects.all()
    pagination_class = PageNumberPagination
    serializer_class = InterceptedRequestModelSerializer
    permission_classes = [IsAuthenticated, SessionOwnerPermission]

    def get_user_sessions(self):
        return InterceptorSession.objects.filter(user=self.request.user).values('id')

    def get_queryset(self):
        """
        Get only intercepted request for sessions created for authenticated user.
        """
        qs = super(InterceptedRequestModelListViewset, self).get_queryset()

        if 'session_id' in self.request.query_params:
            session_id = self.request.query_params.get('session_id')
            return qs.filter(session_id__in=self.get_user_sessions().filter(id=session_id))

        return qs.filter(session_id__in=self.get_user_sessions())


class InterceptorSessionViewset(AuthenticatorMixin, RendererHTMLOnDebugMixin, ModelViewSet):
    queryset = InterceptorSession.objects.all()
    serializer_class = InterceptorSessionSerializer
    pagination_class = PageNumberPagination
    permission_classes = [IsAuthenticated, SessionOwnerPermission]

    def get_queryset(self):
        qs = super(InterceptorSessionViewset, self).get_queryset()
        qs = qs.filter(user=self.request.user)
        return qs

    def perform_create(self, serializer):
        serializer.save(user=self.request.user)



# Generated by Django 3.0.5 on 2020-05-22 01:39

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('interceptor', '0011_interceptorsession_creation_date'),
    ]

    operations = [
        migrations.AddField(
            model_name='interceptedrequest',
            name='matched_response',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='interceptor.InterceptorMockResponse'),
        ),
    ]



# Generated by Django 3.0.5 on 2020-05-14 19:25

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('interceptor', '0008_interceptorsession_session_token'),
    ]

    operations = [
        migrations.CreateModel(
            name='InterceptorMockResponse',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('method', models.CharField(choices=[('get', 'GET'), ('post', 'POST'), ('put', 'PUT'), ('patch', 'PATCH'), ('delete', 'DELETE'), ('head', 'HEAD'), ('options', 'OPTIONS'), ('trace', 'TRACE')], default='get', max_length=8)),
                ('path', models.CharField(default='/', max_length=255)),
                ('status_code', models.PositiveIntegerField()),
                ('response_headers', models.TextField(blank=True, null=True)),
                ('response_body', models.TextField()),
                ('response_description', models.TextField(max_length=255)),
                ('session', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='interceptor.InterceptorSession')),
            ],
        ),
    ]



# Generated by Django 3.0.5 on 2020-05-14 06:17

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import interceptor.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('interceptor', '0004_interceptedfile'),
    ]

    operations = [
        migrations.AddField(
            model_name='interceptedrequest',
            name='timestamp',
            field=models.DateTimeField(auto_now_add=True, null=True),
        ),
        migrations.AlterField(
            model_name='interceptedfile',
            name='file',
            field=models.FileField(blank=True, null=True, upload_to=interceptor.models.session_upload_to),
        ),
        migrations.CreateModel(
            name='InterceptorSession',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('short_name', models.CharField(blank=True, max_length=20)),
                ('active', models.BooleanField(default=True)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'unique_together': {('short_name', 'user')},
            },
        ),
    ]



from rest_framework.permissions import BasePermission

from interceptor.models import InterceptorSession, InterceptedRequest


class SessionOwnerPermission(BasePermission):

    def has_object_permission(self, request, view, obj):
        if isinstance(obj, InterceptorSession):
            return obj.user == request.user
        if isinstance(obj, InterceptedRequest):
            return obj.session.user == request.user
        return False



from rest_framework.routers import DefaultRouter

from interceptor.api.viewsets import InterceptedRequestModelListViewset, InterceptorSessionViewset

router = DefaultRouter()

router.register('request', InterceptedRequestModelListViewset)
router.register('session', InterceptorSessionViewset)

urlpatterns = router.urls



import json

from rest_framework.fields import CharField


class JSONTextField(CharField):
    def to_representation(self, value):
        value = super(JSONTextField, self).to_representation(value)
        try:
            return json.loads(value)
        except json.decoder.JSONDecodeError:
            return {}



# Generated by Django 3.0.5 on 2020-05-14 02:56

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('interceptor', '0003_interceptedrequest_content_type'),
    ]

    operations = [
        migrations.CreateModel(
            name='InterceptedFile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('parameter', models.CharField(max_length=250)),
                ('filename', models.CharField(max_length=250)),
                ('size', models.FloatField(blank=True, null=True)),
                ('file', models.FileField(blank=True, null=True, upload_to='')),
                ('request', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='interceptor.InterceptedRequest')),
            ],
        ),
    ]



# Generated by Django 3.0.5 on 2020-05-18 08:56

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('interceptor', '0010_auto_20200518_0529'),
    ]

    operations = [
        migrations.AddField(
            model_name='interceptorsession',
            name='creation_date',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.0.5 on 2020-05-14 07:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('interceptor', '0007_auto_20200514_0713'),
    ]

    operations = [
        migrations.AddField(
            model_name='interceptorsession',
            name='session_token',
            field=models.CharField(blank=True, max_length=24, null=True),
        ),
    ]



from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from interceptor.api.fields import JSONTextField
from interceptor.models import InterceptedRequest, InterceptorMockResponse, InterceptorSession
from rest_framework.status import HTTP_200_OK


class InterceptorMockResponseSerializer(serializers.ModelSerializer):
    headers = JSONTextField(source='response_headers')
    data = JSONTextField(source='response_body')

    class Meta:
        model = InterceptorMockResponse
        fields = (
            'status_code',
            'headers',
            'data',
        )


class InterceptedRequestModelSerializer(serializers.ModelSerializer):
    data = JSONTextField()
    metadata = JSONTextField()
    params = JSONTextField()
    headers = JSONTextField()
    response = serializers.SerializerMethodField(source='matched_response')

    class Meta:
        model = InterceptedRequest
        fields = (
            'id',
            'path',
            'method',
            'params',
            'data',
            'metadata',
            'created_at',
            'headers',
            'content_type',
            'timestamp',
            'session',
            'response'
        )

    def get_response(self, obj):
        if obj.matched_response:
            data = InterceptorMockResponseSerializer(obj.matched_response).data
            data.update({'match': True})
            return data
        return {
            "match": False,
            "data": {'status': 'You have sent a request, but we encourage you to create your own sessions!'},
            "status_code": HTTP_200_OK,
            "headers": {}
        }


class InterceptorSessionSerializer(serializers.ModelSerializer):
    class Meta:
        model = InterceptorSession
        fields = (
            'id',
            'short_name',
            'requires_authentication',
            'saves_files',
        )

    def validate_short_name(self, data):
        if InterceptorSession.objects.filter(user=self.context['request'].user, short_name=data).exists():
            raise ValidationError(f'A session with name {data} already exists')
        return data



from django.conf import settings
from django.contrib import admin

from interceptor.models import InterceptedRequest, InterceptedFile, InterceptorSession, InterceptorMockResponse


class InterceptedFileTabularInline(admin.TabularInline):
    model = InterceptedFile
    extra = 0

    def has_delete_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request, obj):
        return False


class InterceptedRequestModelAdmin(admin.ModelAdmin):
    inlines = [InterceptedFileTabularInline, ]


admin.site.register(InterceptedRequest, InterceptedRequestModelAdmin)


class InterceptorMockResponseTabularInline(admin.TabularInline):
    model = InterceptorMockResponse


class InterceptorSessionModelAdmin(admin.ModelAdmin):

    list_display = ('short_name', 'active')

    inlines = [InterceptorMockResponseTabularInline]

    actions = ('generate_new_token', )

    def get_list_display(self, request):
        if request.user.is_superuser:
            return ('id', ) + self.list_display + ('user', )
        return self.list_display

    def get_fields(self, request, obj=None):
        fields = ['short_name', 'active', 'requires_authentication', 'saves_files', 'session_token']

        if settings.DEBUG:
            fields.append('user')

        return fields

    def get_readonly_fields(self, request, obj=None):
        return ('session_token', )

    def save_model(self, request, obj, form, change):
        if not settings.DEBUG or obj.user is None:
            obj.user = request.user
        obj.save()

    def get_queryset(self, request):
        qs = super(InterceptorSessionModelAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(user=request.user)

    def generate_new_token(self, request, queryset):
        for session in queryset:
            session.generate_new_token()

    generate_new_token.short_description = 'Generate new session token'


admin.site.register(InterceptorSession, InterceptorSessionModelAdmin)




import string
import random

from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.db import models

from interceptor.choices import HTTP_REQUEST_METHODS
from interceptor import validators


def session_upload_to(instance, filename):

    if instance.request.session is None:
        return f'{instance.request.id}/files/{filename}'

    session = instance.request.session
    return f'{session.user.username}/{session.short_name}/files/{filename}'


class InterceptedRequest(models.Model):
    path = models.CharField(max_length=255, default='/')
    method = models.CharField(max_length=20)
    params = models.TextField(null=True, blank=True)
    data = models.TextField(null=True, blank=True)
    metadata = models.TextField()
    created_at = models.DateTimeField(auto_now_add=True)
    headers = models.TextField(null=True, blank=True)
    content_type = models.CharField(max_length=30, null=True, blank=True)
    timestamp = models.DateTimeField(auto_now_add=True, null=True)
    session = models.ForeignKey('InterceptorSession', null=True, blank=True, on_delete=models.CASCADE,
                                related_name='requests')
    matched_response = models.ForeignKey('InterceptorMockResponse', on_delete=models.SET_NULL, null=True, blank=True)

    def __str__(self):
        return f'[{self.method}] {self.created_at}'


class InterceptedFile(models.Model):
    request = models.ForeignKey(InterceptedRequest, on_delete=models.CASCADE, related_name='files')
    parameter = models.CharField(max_length=250)
    filename = models.CharField(max_length=250)
    size = models.FloatField(null=True, blank=True)
    file = models.FileField(null=True, blank=True, upload_to=session_upload_to)

    def __str__(self):
        return self.filename


class InterceptorSession(models.Model):
    short_name = models.CharField(max_length=20, blank=True)
    user = models.ForeignKey(get_user_model(), on_delete=models.CASCADE, null=True, blank=True)
    active = models.BooleanField(default=True)
    requires_authentication = models.BooleanField(default=False)
    saves_files = models.BooleanField(default=False)
    session_token = models.CharField(max_length=24, null=True, blank=True)
    creation_date = models.DateTimeField(auto_now_add=True)

    class Meta:
        unique_together = ('short_name', 'user')

    def __str__(self):
        return self.short_name

    def clean(self):
        if self.short_name.lower() in ['interceptor', 'console']:
            raise ValidationError(
                f'Word "{self.short_name.capitalize()}" is a reserved word as session. Please select another name'
            )

    def save(self, force_insert=False, force_update=False, using=None,
             update_fields=None):
        is_new = not self.id
        if not self.short_name:
            exists = True
            while exists:
                short_name = ''.join(random.choices(string.ascii_lowercase + string.digits, k=5))
                exists = InterceptorSession.objects.filter(short_name=short_name, user=self.user).exists()

            self.short_name = short_name
        super(InterceptorSession, self).save(force_insert, force_update, using, update_fields)
        if is_new:
            self.generate_new_token()

    def generate_new_token(self):
        equal = True
        new_token = ''

        while equal:
            new_token = ''.join(random.choices(string.ascii_lowercase + string.digits, k=24))
            equal = new_token == self.session_token

        self.session_token = new_token
        self.save()


class InterceptorMockResponse(models.Model):
    session = models.ForeignKey(InterceptorSession, on_delete=models.CASCADE, related_name='mocks')

    # Request descriptor
    method = models.CharField(max_length=8, choices=HTTP_REQUEST_METHODS, default='get')
    path = models.CharField(max_length=255, default='/')

    # Response template
    status_code = models.PositiveIntegerField()
    response_headers = models.TextField(null=True, blank=True,
                                        validators=[validators.JSONFormatValidator(allow_null=True)])
    response_body = models.TextField(validators=[validators.JSONFormatValidator()])

    # Aditionals
    response_description = models.TextField(max_length=255)

    def __str__(self):
        return f'{self.get_method_display()} {self.path} => {self.status_code}'



# Generated by Django 3.0.5 on 2020-05-14 01:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('interceptor', '0002_interceptedrequest_headers'),
    ]

    operations = [
        migrations.AddField(
            model_name='interceptedrequest',
            name='content_type',
            field=models.CharField(blank=True, max_length=30, null=True),
        ),
    ]



# Generated by Django 3.0.5 on 2020-04-16 20:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('interceptor', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='interceptedrequest',
            name='headers',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from setuptools import setup, find_packages

setup(
    name="django-thaad",
    version='1.0.3',
    author="Luis Moncaris",
    author_email="lmoncarisg@gmail.com",
    description="Provide utils to intercept and save/redirect requests",
    long_description=open("README.md").read(),
    long_description_content_type='text/markdown',
    packages=find_packages('.', exclude=['test', '*.test', 'docs', 'example', 'media']),
    include_package_data=True,
    install_requires=[
        'Django>=3.0',
        'djangorestframework==3.11.0',
        'django-cors-headers==3.3.0'
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Framework :: Django :: 3.0",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Framework :: Django",
    ],
    zip_safe=False
)




# Generated by Django 3.0.5 on 2020-04-16 20:17

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='InterceptedRequest',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('method', models.CharField(max_length=20)),
                ('params', models.TextField(blank=True, null=True)),
                ('data', models.TextField(blank=True, null=True)),
                ('metadata', models.TextField()),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]



# Generated by Django 3.0.5 on 2020-05-14 07:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('interceptor', '0006_auto_20200514_0637'),
    ]

    operations = [
        migrations.AddField(
            model_name='interceptorsession',
            name='requires_authentication',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='interceptorsession',
            name='saves_files',
            field=models.BooleanField(default=False),
        ),
    ]



from django.conf.urls import url
from django.urls import include

from interceptor.api import API_VERSION
from interceptor.views import HTTPInterceptorView, HTTPSessionInterceptorView

app_name = 'interceptor'

urlpatterns = [
    url(f'^api/{API_VERSION}/', include('interceptor.api.urls')),
    url('^interceptor/(.*)', HTTPInterceptorView.as_view()),
    url(r'^s/(?P<session_name>[A-Za-z0-9]{1,20})/(.*)', HTTPSessionInterceptorView.as_view())  # Should remain last
]
